@@ -103,9 +103,11 @@ int Assemble(char *mddev, int mdfd,
 		long long events;
 		time_t utime;
 		int uptodate;
+		int state;
 		int raid_disk;
 	} *devices;
-	int *best; /* indexed by raid_disk */
+	int *best = NULL; /* indexed by raid_disk */
+	int bestcnt = 0;
 	int devcnt = 0, okcnt, sparecnt;
 	int i;
 	int most_recent = 0;
@@ -183,8 +185,11 @@ int Assemble(char *mddev, int mdfd,
 		devlist = devlist->next;
 
 		if (ident->devices &&
-		    !match_oneof(ident->devices, devname))
+		    !match_oneof(ident->devices, devname)) {
+			if (inargv || verbose)
+				fprintf(stderr, Name ": %s is not one of %s\n", devname, ident->devices);
 			continue;
+		}
 		
 		dfd = open(devname, O_RDONLY, 0);
 		if (dfd < 0) {
@@ -310,6 +315,7 @@ int Assemble(char *mddev, int mdfd,
 		devices[devcnt].utime = super.utime;
 		devices[devcnt].raid_disk = super.this_disk.raid_disk;
 		devices[devcnt].uptodate = 0;
+		devices[devcnt].state = super.this_disk.state;
 		if (most_recent < devcnt) {
 			if (devices[devcnt].events
 			    > devices[most_recent].events)
@@ -320,11 +326,24 @@ int Assemble(char *mddev, int mdfd,
 			i = devcnt;
 		else
 			i = devices[devcnt].raid_disk;
-		if (i>=0 && i < num_devs)
+		if (i>=0 && i < 10000) {
+			if (i >= bestcnt) {
+				int newbestcnt = i+10;
+				int *newbest = malloc(sizeof(int)*newbestcnt);
+				int c;
+				for (c=0; c < newbestcnt; c++)
+					if (c < bestcnt)
+						newbest[c] = best[c];
+					else
+						newbest[c] = -1;
+				if (best)free(best);
+				best = newbest;
+				bestcnt = newbestcnt;
+			}
 			if (best[i] == -1
 			    || devices[best[i]].events < devices[devcnt].events)
 				best[i] = devcnt;
-
+		}
 		devcnt++;
 	}
 
@@ -338,10 +357,16 @@ int Assemble(char *mddev, int mdfd,
 	 */
 	okcnt = 0;
 	sparecnt=0;
-	for (i=0; i< num_devs ;i++) {
+	for (i=0; i< bestcnt ;i++) {
 		int j = best[i];
 		int event_margin = !force;
 		if (j < 0) continue;
+		/* note: we ignore error flags in multipath arrays
+		 * as they don't make sense
+		 */
+		if (first_super.level != -4)
+			if (!(devices[j].state & (1<<MD_DISK_SYNC)))
+				continue;
 		if (devices[j].events+event_margin >=
 		    devices[most_recent].events) {
 			devices[j].uptodate = 1;
@@ -358,7 +383,7 @@ int Assemble(char *mddev, int mdfd,
 		 */
 		int fd;
 		chosen_drive = -1;
-		for (i=0; i<first_super.raid_disks; i++) {
+		for (i=0; i<first_super.raid_disks && i < bestcnt; i++) {
 			int j = best[i];
 			if (j>=0 &&
 			    !devices[j].uptodate &&
@@ -391,7 +416,7 @@ int Assemble(char *mddev, int mdfd,
 		super.events_lo = (devices[most_recent].events)&0xFFFFFFFF;
 		if (super.level == 5 || super.level == 4) {
 			/* need to force clean */
-			super.state = 0;
+			super.state = (1<<MD_SB_CLEAN);
 		}
 		super.sb_csum = calc_sb_csum(&super);
 /*DRYRUN*/	if (store_super(fd, &super)) {
@@ -414,7 +439,7 @@ int Assemble(char *mddev, int mdfd,
 	 * superblock.
 	 */
 	chosen_drive = -1;
-	for (i=0; chosen_drive < 0 && i<num_devs; i++) {
+	for (i=0; chosen_drive < 0 && i<bestcnt; i++) {
 		int j = best[i];
 		int fd;
 		if (j<0)
@@ -436,7 +461,7 @@ int Assemble(char *mddev, int mdfd,
 		close(fd);
 	}
 
-	for (i=0; i<num_devs; i++) {
+	for (i=0; i<bestcnt; i++) {
 		int j = best[i];
 		int desired_state;
 
@@ -484,6 +509,11 @@ This doesnt work yet
 				i, mddev);
 		}
 	}
+	if (force && (super.level == 4 || super.level == 5) && 
+	    okcnt == super.raid_disks-1) {
+		super.state = (1<< MD_SB_CLEAN);
+		change |= 2;
+	}
 
 	if ((force && (change & 2))
 	    || (old_linux && (change & 1))) {
@@ -513,16 +543,16 @@ This doesnt work yet
 			return 1;
 		}
 		/* First, add the raid disks, but add the chosen one last */
-		for (i=0; i<= num_devs; i++) {
+		for (i=0; i<= bestcnt; i++) {
 			int j;
-			if (i < num_devs) {
+			if (i < bestcnt) {
 				j = best[i];
 				if (j == chosen_drive)
 					continue;
 			} else
 				j = chosen_drive;
 
-			if (j >= 0 && devices[j].uptodate) {
+			if (j >= 0 /* && devices[j].uptodate */) {
 				mdu_disk_info_t disk;
 				memset(&disk, 0, sizeof(disk));
 				disk.major = devices[j].major;
